use std::fmt::Display;

use bitvec::{order::Msb0, vec::BitVec};

#[derive(Debug)]
pub enum Expr {
    Abs(Box<Expr>),
    App(Box<Expr>, Box<Expr>),
    Var(usize),
    DefinitionName(String),
}

#[derive(Debug)]
pub enum FlattenedExpr {
    Abs(Box<FlattenedExpr>),
    App(Box<FlattenedExpr>, Box<FlattenedExpr>),
    Var(usize),
}

impl FlattenedExpr {
    pub fn to_blc(&self, bv: &mut BitVec<u8, Msb0>) {
        match self {
            FlattenedExpr::Abs(expr) => {
                bv.push(false);
                bv.push(false);

                expr.to_blc(bv);
            }
            FlattenedExpr::App(left, right) => {
                bv.push(false);
                bv.push(true);

                left.to_blc(bv);
                right.to_blc(bv);
            }
            FlattenedExpr::Var(x) => {
                for _ in 0..=*x {
                    bv.push(true);
                }

                bv.push(false);
            }
        }
    }
}

impl Display for FlattenedExpr {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            FlattenedExpr::Abs(expr) => {
                write!(f, "λ{}", expr)?;
            }
            FlattenedExpr::App(left, right) => write!(f, "[{} {}]", left, right)?,
            FlattenedExpr::Var(var) => write!(f, "{var}")?,
        }

        Ok(())
    }
}
