use std::{fs, io::Write};

use bitvec::vec::BitVec;
use flatten::Flattener;
use parser::Parser;
use scanner::Scanner;

mod expr;
mod flatten;
mod parser;
mod scanner;
mod stmt;
mod token;

fn main() -> anyhow::Result<()> {
    let args: Vec<_> = std::env::args().collect();
    if args.len() != 2 {
        println!("Usage: {} filename.sos", args[0]);
        return Ok(());
    }

    let code = fs::read_to_string(&args[1])?;

    run(&code);

    Ok(())
}

fn run(code: &str) -> Option<()> {
    let scanner = Scanner::new(code);

    let tokens = scanner.scan_tokens()?;

    let mut parser = Parser::new(tokens);
    let (funcs, main) = parser.parse()?;

    let mut flattener = Flattener::new(funcs);
    let flat = flattener.flatten(&main)?;

    let mut bv = BitVec::new();
    flat.to_blc(&mut bv);

    let mut stdout = std::io::stdout();
    for x in bv.as_raw_slice() {
        stdout.write_all(&[*x]).unwrap();
    }

    Some(())
}
